#ifndef ALLOCATOR_H
#define ALLOCATOR_H

#include "Utils.h"

class MemMediaSample : public IMediaSample {
	MemMediaSample(const MemMediaSample&);
	MemMediaSample& operator=(const MemMediaSample&);
public:
	MemMediaSample();
	~MemMediaSample();

	HRESULT Init(IMemAllocator *pAlloc, const ALLOCATOR_PROPERTIES& props);

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE GetPointer(BYTE **ppBuffer);
	long STDMETHODCALLTYPE GetSize();
	HRESULT STDMETHODCALLTYPE GetTime(REFERENCE_TIME *pTimeStart, REFERENCE_TIME *pTimeEnd);
	HRESULT STDMETHODCALLTYPE SetTime(REFERENCE_TIME *pTimeStart, REFERENCE_TIME *pTimeEnd);        
	HRESULT STDMETHODCALLTYPE IsSyncPoint();
	HRESULT STDMETHODCALLTYPE SetSyncPoint(BOOL bIsSyncPoint);
	HRESULT STDMETHODCALLTYPE IsPreroll();
	HRESULT STDMETHODCALLTYPE SetPreroll(BOOL bIsPreroll);
	long STDMETHODCALLTYPE GetActualDataLength();
	HRESULT STDMETHODCALLTYPE SetActualDataLength(long Length);
	HRESULT STDMETHODCALLTYPE GetMediaType(AM_MEDIA_TYPE **ppMediaType);
	HRESULT STDMETHODCALLTYPE SetMediaType(AM_MEDIA_TYPE *pMediaType);
	HRESULT STDMETHODCALLTYPE IsDiscontinuity();
	HRESULT STDMETHODCALLTYPE SetDiscontinuity(BOOL bDiscontinuity);
	HRESULT STDMETHODCALLTYPE GetMediaTime(LONGLONG *pTimeStart, LONGLONG *pTimeEnd);
	HRESULT STDMETHODCALLTYPE SetMediaTime(LONGLONG *pTimeStart, LONGLONG *pTimeEnd);

protected:
	LONG	mRefCount;
	IMemAllocator *mpAllocator;

	bool		mbIsPreroll;
	bool		mbIsSyncPoint;
	bool		mbIsDiscontinuity;
	bool		mbStreamStartTimeSet;
	bool		mbStreamStopTimeSet;
	bool		mbMediaTimesSet;
	long		mDataLength;
	void		*mpBuffer;
	void		*mpBufferBase;
	long		mBufferSize;
	LONGLONG	mMediaTimeStart;
	LONGLONG	mMediaTimeEnd;
	REFERENCE_TIME	mStreamTimeStart;
	REFERENCE_TIME	mStreamTimeEnd;
	MediaTypePtr	mpMediaType;
};

class MemAllocator : public IMemAllocator {
	MemAllocator(const MemAllocator&);
	MemAllocator& operator=(const MemAllocator&);
public:
	MemAllocator();
	~MemAllocator();

	HRESULT Init();

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE SetProperties(ALLOCATOR_PROPERTIES *pRequest, ALLOCATOR_PROPERTIES *pActual);
	HRESULT STDMETHODCALLTYPE GetProperties(ALLOCATOR_PROPERTIES *pProps);
	HRESULT STDMETHODCALLTYPE Commit();
	HRESULT STDMETHODCALLTYPE Decommit();
	HRESULT STDMETHODCALLTYPE GetBuffer(IMediaSample **ppBuffer, REFERENCE_TIME *pStartTime, REFERENCE_TIME *pEndTime, DWORD dwFlags);
	HRESULT STDMETHODCALLTYPE ReleaseBuffer(IMediaSample *pBuffer);

protected:
	Lock	mLock;
	HANDLE	mBufferAvailable;
	bool	mbCommitted;
	int		mWaiters;
	int		mBuffersOutstanding;

	std::vector<MemMediaSample *> mSamples;

	ALLOCATOR_PROPERTIES	mProps;

	LONG	mRefCount;
};

#endif
