#ifndef BASEFILTER_H
#define BASEFILTER_H

#include "Utils.h"
#include "BasePin.h"

class BaseFilter : public IBaseFilter {
	BaseFilter(const BaseFilter&);
	BaseFilter& operator=(const BaseFilter&);

	friend class BaseFilterPinEnum;
public:
	BaseFilter();
	~BaseFilter();

	void ReservePins(unsigned pins);
	void AddPin(BasePin *pin, LPCWSTR id);

	virtual bool IsStopped();

	IFilterGraph *GetGraph() { return mpGraph; }
	BSTR GetNameInGraph() { return mpNameInGraph; }

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE EnumPins(IEnumPins **ppEnum);
	HRESULT STDMETHODCALLTYPE FindPin(LPCWSTR Id, IPin **ppPin);
	HRESULT STDMETHODCALLTYPE QueryFilterInfo(FILTER_INFO *pInfo);
	HRESULT STDMETHODCALLTYPE JoinFilterGraph(IFilterGraph *pGraph, LPCWSTR pName);
	HRESULT STDMETHODCALLTYPE QueryVendorInfo(LPWSTR *pVendorInfo);

	HRESULT STDMETHODCALLTYPE Stop();
	HRESULT STDMETHODCALLTYPE Pause();
	HRESULT STDMETHODCALLTYPE Run(REFERENCE_TIME tStart);
	HRESULT STDMETHODCALLTYPE GetState(DWORD dwMilliSecsTimeout, FILTER_STATE *State);
	HRESULT STDMETHODCALLTYPE SetSyncSource(IReferenceClock *pClock);
	HRESULT STDMETHODCALLTYPE GetSyncSource(IReferenceClock **pClock);

protected:
	IFilterGraph *mpGraph;
	BSTR mpNameInGraph;
	LONG mRefCount;

	typedef std::vector<BasePin *> PinSet;
	PinSet mPinSet;
	int	mPinSetGeneration;

	Lock mLock;
	IReferenceClock *mpClock;
	FILTER_STATE mState;
};

#endif
