#ifndef DSENGINE_H
#define DSENGINE_H

#ifdef _MSC_VER
	#pragma once
#endif

#include <vd2/system/refcount.h>
#include <vd2/system/vdstl.h>
#include <vd2/system/error.h>
#include <vd2/system/thread.h>

class DSEngineCommand : public vdlist_node {
public:
	DSEngineCommand(VDSignal *signal) : mpCallback(NULL), mpCallbackData(NULL), mpData(NULL), mpSignal(signal->getHandle() ? signal : NULL), mCompleted(0) {}

	template<class T, void (T::*Method)(void *data)>
	static void MethodAdapter(void *data, void *cbdata) {
		(((T *)cbdata)->*Method)( data);
	}

	template<class T, void (T::*Method)(void *data)>
	void SetMethodCallback(T *pThis) {
		mpCallback = MethodAdapter<T, Method>;
		mpCallbackData = pThis;
	}

	void SetExtraData(void *p) {
		mpData = p;
	}

	void Reset() {
		mCompleted = 0;
		mError.clear();
	}

	void ThrowError() {
		if (!mError.empty())
			throw mError;
	}

	void (*mpCallback)(void *data, void *cbdata);
	void *mpCallbackData;
	void *mpData;

	VDSignal		*mpSignal;
	VDAtomicInt		mCompleted;
	MyError			mError;
};

class IDSEngine : public IVDRefCount {
public:
	virtual bool Init() = 0;
	virtual void Shutdown() = 0;

	virtual void PostCommand(DSEngineCommand *cmd) = 0;
	virtual void WaitCommand(DSEngineCommand *cmd) = 0;
	virtual void SendCommand(DSEngineCommand *cmd) = 0;
	virtual void Finish(VDSignal *) = 0;
};

bool VDCreateDSEngine(IDSEngine **);

#endif
