#ifndef SAMPLEGRABBER_H
#define SAMPLEGRABBER_H

#include <vd2/system/thread.h>
#include <vd2/system/vdstl.h>
#include "BaseFilter.h"
#include "TransformPin.h"

class IDSSampleGrabberCallback {
public:
	virtual bool Process(IMediaSample *pSample, const REFERENCE_TIME& segmentStart) = 0;
};

class DSSampleGrabberInputPin : public TransformInputPin {
public:
	DSSampleGrabberInputPin();

	VDSignal& GetSignal() { return mSignal; }

	void SetRunning(bool running);

	bool CheckForSample(const REFERENCE_TIME& tstart, const REFERENCE_TIME& tend);
	void ResetOneShot(const REFERENCE_TIME& tstart, const REFERENCE_TIME& tend);
	bool ResumeOneShot(const REFERENCE_TIME& tstart, const REFERENCE_TIME& tend);
	bool GetEstimatedStreamPosition(REFERENCE_TIME *pos);
	void SetCallback(IDSSampleGrabberCallback *pCB);
	void SetMediaType(const GUID *type);
	void SetMediaSubType(const GUID *subtype);

	HRESULT STDMETHODCALLTYPE QueryAccept(const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE Receive(IMediaSample *pSample);
	HRESULT STDMETHODCALLTYPE BeginFlush();
	HRESULT STDMETHODCALLTYPE EndOfStream();
	HRESULT STDMETHODCALLTYPE NewSegment(REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate);

protected:
	void ClearQueueLocked();

	GUID mMediaType;
	GUID mSubType;
	REFERENCE_TIME mTargetTimeStart;
	REFERENCE_TIME mTargetTimeEnd;
	REFERENCE_TIME mTimeBase;
	REFERENCE_TIME mLastSeenTime;
	REFERENCE_TIME mLastSeenTimeSnapshot;
	bool mbRunning;
	bool mbBufferSamples;
	bool mbSampleCaptured;
	bool mbRestartOnNewSegment;
	IDSSampleGrabberCallback *mpCB;
	VDSignal mSignal;

	struct SampleEntry {
		IMediaSample *mpSample;
		REFERENCE_TIME	mStartTime;
		REFERENCE_TIME	mEndTime;
	};

	typedef vdfastvector<SampleEntry> MediaSamples;
	MediaSamples mMediaSamples;
};

class DSSampleGrabber : public BaseFilter {
public:
	DSSampleGrabber();
	~DSSampleGrabber();

	BasePin *GetInputPin() { return &mInputPin; }
	BasePin *GetOutputPin() { return &mOutputPin; }

	void SetCallback(IDSSampleGrabberCallback *pCB);
	void SetMediaType(const GUID *type);
	void SetMediaSubType(const GUID *subtype);
	bool GetEstimatedStreamPosition(REFERENCE_TIME *pos);
	bool CheckForSample(const REFERENCE_TIME& tstart, const REFERENCE_TIME& tend);
	void ResetOneShot(const REFERENCE_TIME& tstart, const REFERENCE_TIME& tend);
	bool ResumeOneShot(const REFERENCE_TIME& tstart, const REFERENCE_TIME& tend);
	void WaitForSample();

	HRESULT STDMETHODCALLTYPE GetClassID(CLSID *pClassID);
	HRESULT STDMETHODCALLTYPE Stop();
	HRESULT STDMETHODCALLTYPE Pause();
	HRESULT STDMETHODCALLTYPE Run(REFERENCE_TIME tStart);

protected:
	DSSampleGrabberInputPin mInputPin;
	TransformOutputPin	mOutputPin;
};

#endif
