#ifndef TRANSFORMPIN_H
#define TRANSFORMPIN_H

#include "MemPin.h"

class TransformOutputPin;

class TransformInputPin : public MemInputPin {
public:
	TransformInputPin();
	~TransformInputPin();

	void SetAssociatedOutput(TransformOutputPin *pAssociatedOutput);

	HRESULT STDMETHODCALLTYPE ReceiveConnection(IPin *pConnector, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryAccept(const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryInternalConnections(IPin **apPin, ULONG *nPin);
	HRESULT STDMETHODCALLTYPE BeginFlush();
	HRESULT STDMETHODCALLTYPE EndFlush();
	HRESULT STDMETHODCALLTYPE EndOfStream();
	HRESULT STDMETHODCALLTYPE NewSegment(REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate);

protected:
	TransformOutputPin *mpAssocOutputPin;
	bool mbFlushing;
};

class TransformOutputPin : public MemOutputPin, public IMediaSeeking {
public:
	TransformOutputPin();
	~TransformOutputPin();

	TransformInputPin *GetAssociatedInput() { return mpAssocInputPin; }
	void SetAssociatedInput(TransformInputPin *pAssociatedInput);

	void Output(IMediaSample *sample);

	void NotifyNewInputFormat();

	// IUnknown
	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	// IPin
	HRESULT STDMETHODCALLTYPE QueryInternalConnections(IPin **apPin, ULONG *nPin);
	HRESULT STDMETHODCALLTYPE EnumMediaTypes(IEnumMediaTypes **ppEnum);

	// IMediaSeeking
	HRESULT STDMETHODCALLTYPE GetCapabilities(DWORD *pCapabilities);
	HRESULT STDMETHODCALLTYPE CheckCapabilities(DWORD *pCapabilities);
	HRESULT STDMETHODCALLTYPE IsFormatSupported(const GUID *pFormat);
	HRESULT STDMETHODCALLTYPE QueryPreferredFormat(GUID *pFormat);
	HRESULT STDMETHODCALLTYPE GetTimeFormat(GUID *pFormat);
	HRESULT STDMETHODCALLTYPE IsUsingTimeFormat(const GUID *pFormat);
	HRESULT STDMETHODCALLTYPE SetTimeFormat(const GUID *pFormat);
	HRESULT STDMETHODCALLTYPE GetDuration(LONGLONG *pDuration);
	HRESULT STDMETHODCALLTYPE GetStopPosition(LONGLONG *pStop);
	HRESULT STDMETHODCALLTYPE GetCurrentPosition(LONGLONG *pCurrent);
	HRESULT STDMETHODCALLTYPE ConvertTimeFormat(LONGLONG *pTarget, const GUID *pTargetFormat, LONGLONG Source, const GUID *pSourceFormat);
	HRESULT STDMETHODCALLTYPE SetPositions(LONGLONG *pCurrent, DWORD dwCurrentFlags, LONGLONG *pStop, DWORD dwStopFlags);
	HRESULT STDMETHODCALLTYPE GetPositions(LONGLONG *pCurrent, LONGLONG *pStop);
	HRESULT STDMETHODCALLTYPE GetAvailable(LONGLONG *pEarliest, LONGLONG *pLatest);
	HRESULT STDMETHODCALLTYPE SetRate(double dRate);
	HRESULT STDMETHODCALLTYPE GetRate(double *pdRate);
	HRESULT STDMETHODCALLTYPE GetPreroll(LONGLONG *pllPreroll);

protected:
	friend class TransformOutputMediaTypeEnumerator;

	bool GetUpstreamMediaSeeking(IMediaSeeking **ppMS);

	TransformInputPin *mpAssocInputPin;

	int mFormatGeneration;
};

#endif
