#ifndef VIDEOSOURCEDSHOW_H
#define VIDEOSOURCEDSHOW_H

#include <vd2/plugin/vdinputdriver.h>
#include <vd2/VDXFrame/Unknown.h>
#include <vd2/system/vdstl.h>
#include <vd2/system/VDString.h>
#include "Engine.h"
#include "SampleGrabber.h"

class DSVideoDecoder : public vdxunknown<IVDXVideoDecoder>, public IDSSampleGrabberCallback {
public:
	DSVideoDecoder(const VDXInputDriverContext *context, IDSEngine *engine, const wchar_t *fn);
	~DSVideoDecoder();

	const VDXStreamSourceInfo&	GetStreamInfo() const { return mStreamInfo; }
	const VDXVideoSourceInfo&	GetVideoInfo() const { return mVideoInfo; }

	const void *		VDXAPIENTRY DecodeFrame(const void *inputBuffer, uint32 data_len, bool is_preroll, sint64 sampleNumber, sint64 targetFrame);
	uint32				VDXAPIENTRY GetDecodePadding();
	void				VDXAPIENTRY Reset();
	bool				VDXAPIENTRY IsFrameBufferValid();
	const VDXPixmap&	VDXAPIENTRY GetFrameBuffer();
	bool				VDXAPIENTRY SetTargetFormat(int format, bool useDIBAlignment);
	bool				VDXAPIENTRY SetDecompressedFormat(const VDXBITMAPINFOHEADER *pbih);

	bool				VDXAPIENTRY IsDecodable(sint64 sample_num);
	const void *		VDXAPIENTRY GetFrameBufferBase();

protected:
	void InitLocal(void *pvFn);
	void ShutdownLocal(void *);
	void DecodeFrameLocal(void *pvFrame);
	void SetFormatLocal(void *pvFormat);
	bool Process(IMediaSample *pSample, const REFERENCE_TIME& timeBase);

	vdrefptr<IDSEngine>			mpEngine;
	const VDXInputDriverContext *const mpContext;

	vdrefptr<IGraphBuilder>		mpGraph;
	vdrefptr<IMediaControl>		mpGraphMediaControl;
	vdrefptr<IMediaEvent>		mpGraphMediaEvent;
	vdrefptr<IMediaSeeking>		mpGraphMediaSeeking;
	vdrefptr<IMediaFilter>		mpGraphMediaFilter;
	vdrefptr<IPin>				mpSourcePin;
	vdrefptr<IPin>				mpRendererPin;
	vdrefptr<DSSampleGrabber>	mpSampleGrabber;
	vdrefptr<IBaseFilter>		mpRenderer;

	VDXStreamSourceInfo			mStreamInfo;
	VDXVideoSourceInfo			mVideoInfo;

	uint32		mFormatSupportMask;
	bool		mbFrameBufferValid;

	DWORD		mGraphRegistration;

	VDSignal	mCommandSignal;
	VDXPixmap mpxFrameBuffer;
	vdfastvector<uint32> mFrameBuffer;
};

class DSVideoSource : public vdxunknown<IVDXVideoSource>, public IVDXStreamSource, public IVDXStreamSourceV3 {
public:
	DSVideoSource(const VDXInputDriverContext *context, IDSEngine *engine, const wchar_t *fn);
	~DSVideoSource();

	int		VDXAPIENTRY AddRef();
	int		VDXAPIENTRY Release();
	void *	VDXAPIENTRY AsInterface(uint32 id);

	void				VDXAPIENTRY GetStreamSourceInfo(VDXStreamSourceInfo&);

	bool				VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);

	const void *		VDXAPIENTRY GetDirectFormat();
	int					VDXAPIENTRY GetDirectFormatLen();

	ErrorMode			VDXAPIENTRY GetDecodeErrorMode();
	void				VDXAPIENTRY SetDecodeErrorMode(ErrorMode mode);
	bool				VDXAPIENTRY IsDecodeErrorModeSupported(ErrorMode mode);

	bool				VDXAPIENTRY IsVBR();
	sint64				VDXAPIENTRY TimeToPositionVBR(sint64 us);
	sint64				VDXAPIENTRY PositionToTimeVBR(sint64 samples);

	// IVDXStreamSourceV3
	void	VDXAPIENTRY GetStreamSourceInfoV3(VDXStreamSourceInfoV3&);

	// IVDXVideoSource
	void	VDXAPIENTRY GetVideoSourceInfo(VDXVideoSourceInfo& info);

	bool	VDXAPIENTRY CreateVideoDecoderModel(IVDXVideoDecoderModel **);
	bool	VDXAPIENTRY CreateVideoDecoder(IVDXVideoDecoder **);

	void	VDXAPIENTRY GetSampleInfo(sint64 sample_num, VDXVideoFrameInfo& frameInfo);

	bool	VDXAPIENTRY IsKey(sint64 sample_num);

	sint64	VDXAPIENTRY GetFrameNumberForSample(sint64 sample_num);
	sint64	VDXAPIENTRY GetSampleNumberForFrame(sint64 frame_num);
	sint64	VDXAPIENTRY GetRealFrame(sint64 frame_num);

	sint64	VDXAPIENTRY GetSampleBytePosition(sint64 sample_num);

protected:
	vdrefptr<DSVideoDecoder> mpCachedDecoder;

	VDXStreamSourceInfo		mStreamInfo;
	VDXVideoSourceInfo		mVideoInfo;

	vdrefptr<IDSEngine> mpEngine;
	const VDXInputDriverContext *const mpContext;
	VDStringW mFilename;
};

#endif
